//Created by Amin Rahmani, Ph.D., P.E. - February 6, 2018
//Copyright © 2018 Amin Rahmani. All rights reserved.

//==========================================================================================================================
//                                '****   Notes  ****
// This ".cpp" file gets input values from user, reads input CPT file (in text format), calculates Ic per Robertson and Wride (1998), and creates output file.
// The analysis is performed based on CPT data which must be in Imperial units : depth (ft), cone resistance and sleeve friction in (tsf) and water pressure in (psi).
// This file is used by files "B&I(2014)" and "Youd_et_al(2001)".
//===========================================================================================================================

#include "basicIntrep.hpp"

basicIntrep::basicIntrep () {
    //Do nothing
}

basicIntrep::~basicIntrep () {
    //Do nothing
}

//Gets the input parameters for analysis
void basicIntrep :: getValues (double insituWT, double historWT, double PGA, double Mw, double Ic_cutoff, string CPTdata, string outputs) {

    this-> insituWT = insituWT;
    this-> historWT = historWT;
    this-> PGA = PGA;
    this-> Mw = Mw;
    this-> Ic_cutoff = Ic_cutoff;
    this-> CPTdata = CPTdata;
    this-> outputs = outputs;
}

//Reads CPT data
void basicIntrep :: readCPT () {

    ifstream inputData;
    inputData.open (CPTdata);

    double col1, col2, col3, col4;

    numDataPoint=0;

    if ( inputData.fail()) {
        cerr << "Error in opening the input file" << endl;
        exit(1);
    } else {
        while (! inputData.eof() ) {
            inputData >> col1 >> col2  >> col3 >> col4;
            depth.push_back(col1);
            q_c.push_back(col2);
            f_s.push_back(col3);
            u_water.push_back(col4);
            numDataPoint++;
        }
        inputData.close();
    }
}

void basicIntrep :: basicCalcs () {


    for (int i=0; i < numDataPoint; i++) {

        //Corrected cone resistance
        q_t.push_back(q_c[i]+(u_water[i]*144/2000)*(1-a));

        //soil unit weight
        gamma_t.push_back((0.27*log10(f_s[i]/q_t[i]*100)+0.36*log10(q_t[i]*2000.0/2089)+1.236)*62.4);

        //Insitu stresses
        TotStress.push_back(gamma_t[i]*depth[i]/2000.0);

        if (depth[i] <= insituWT) {
            insitu_hydr.push_back(0.0);
        } else {
            insitu_hydr.push_back(62.4*(depth[i]-insituWT)/2000.0);
        }

        insitu_EffStress.push_back(TotStress[i]-insitu_hydr[i]);

        //Design stresses
        if (depth[i] <= historWT) {
            design_hydr.push_back(0.0);
        } else {
            design_hydr.push_back(62.4*(depth[i]-historWT)/2000.0);
        }

        design_EffStress.push_back(TotStress[i]-design_hydr[i]);

        //Calculate Ic
        n_pow.push_back(1.0);
        double new_n_pow =0.5;

        CN.push_back(min(2.0,pow(1.058/insitu_EffStress[i],n_pow[i])));
        Qn.push_back((q_t[i]-TotStress[i])*CN[i]/1.058);
        Fn.push_back(f_s[i]*100.0/(q_t[i]-TotStress[i]));
        Ic.push_back(sqrt(pow((3.47-log10 (Qn[i])),2)+pow((1.22+log10 (Fn[i])),2)));

        // iteration to calculate Ic
        while ( abs(n_pow[i]-new_n_pow) > 0.01) {

            replace( CN.begin(), CN.end(), CN[i], min(2.0,pow(1.058/insitu_EffStress[i],n_pow[i])) );
            replace( Qn.begin(), Qn.end(), Qn[i], (q_t[i]-TotStress[i])*CN[i]/1.058 );
            replace( Ic.begin(), Ic.end(), Ic[i], sqrt(pow((3.47-log10 (Qn[i])),2)+pow((1.22+log10 (Fn[i])),2)) );

            new_n_pow=n_pow[i];

            //Iteration per Robertson & Wride (1998)
            if (Ic[i] <= 1.64) {
                replace( n_pow.begin(), n_pow.end(), n_pow[i], 0.5);
            } else if (Ic[i] <= 3.3) {
                replace( n_pow.begin(), n_pow.end(), n_pow[i], (Ic[i]-1.64)*0.3+0.5);
            } else {
                replace( n_pow.begin(), n_pow.end(), n_pow[i], 1.0);
            }

        }
    }

}

// Recording outputs
void basicIntrep::writeOutputs() {

        ofstream outputData;
        outputData.open (outputs);

        outputData << "Depth(ft)" << " " << "qt(tsf)" << " " << "fs(tsf)" << " " << "qc1N_cs" << " " << "Ic" << " " <<  "CRR" << " " << "CSR" << " " << "FoS" << endl;

        for (int i=0; i < numDataPoint; i++) {
            outputData << depth[i] << " " << q_t[i] << " " << f_s[i] << " " << qc1N_cs[i] << " " << Ic[i] << " " <<  CRR[i] << " " << CSR [i] << " " << FoS[i] << endl;
        }

outputData.close();

}
